﻿using Hotline.SeedData;
using SqlSugar;
using System.ComponentModel;
using XF.Domain.Exceptions;
using XF.Domain.Repository;
using XF.Utility.EnumExtensions;

namespace Hotline.Settings;

[SugarIndex("unique_org_code", nameof(SystemOrganize.OrgCode), OrderByType.Desc, true)]
[Description("组织架构")]
public class SystemOrganize : CreationSoftDeleteEntity
{
    /// <summary>
    /// 组织架构名称
    /// </summary>
    public string OrgName { get; set; }

    /// <summary>
    /// 组织架构简称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string OrgShortName { get; set; }

    /// <summary>
    /// 组织架构Code（001001001） 解析：001,001,001）
    /// </summary>
    public string OrgCode { get; set; }

    /// <summary>
    /// 区域Code（行政区域代码）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string AreaCode { get; set; }

    /// <summary>
    /// 区域名称（行政区域名称）
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string AreaName { get; set; }

    /// <summary>
    /// 部门级别
    /// </summary>
    public int OrgLevel { get; set; }

    /// <summary>
    /// 部门类型
    /// </summary>
    public EOrgType OrgType { get; set; }

    [SugarColumn(IsIgnore = true)]
    public string OrgTypeText => OrgType.GetDescription();

    /// <summary>
    /// 上级ID
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ParentId { get; set; }

    /// <summary>
    /// 上级名称
    /// </summary>
    [SugarColumn(IsNullable = true)]
    public string? ParentName { get; set; }

    /// <summary>
    /// 是否启用
    /// </summary>
    public bool IsEnable { get; set; }

    [SugarColumn(IsIgnore = true)]
    public List<SystemOrganize> Children { get; set; }


    public void InitOrgLevel() => OrgLevel = OrgCode.CalcOrgLevel();
}

public static class OrgExtensions
{
    public static int CalcOrgLevel(this string orgCode)
    {
        if (orgCode.Length % 3 != 0)
            throw new UserFriendlyException("非法部门Code");
        //return (orgCode.Length / 3) - 1;
        return orgCode.Length / 3;
    }

    public static bool CheckIfOrgLevelIs(this string orgCode, int orgLevel)
    {
        ////中心算做一级部门
        //if (orgLevel == 1 && orgCode == OrgSeedData.CallCenterCode)
        //    return true;

        var level = orgCode.CalcOrgLevel();
        return level == orgLevel;
    }

    /// <summary>
    /// 根据orgCode获取该部门所属对应级别的上级部门code
    /// </summary>
    /// <param name="orgCode"></param>
    /// <param name="orgLevel">默认一级部门</param>
    /// <returns></returns>
    public static string GetUpperOrgCode(this string orgCode, int orgLevel = 1)
    {
        if (string.IsNullOrEmpty(orgCode))
            throw UserFriendlyException.SameMessage("无效部门编码");
        ////中心算做一级部门
        //if (orgLevel == 1 && orgCode == OrgSeedData.CallCenterCode) return orgCode;

        return orgCode.Substring(0, 3 * orgLevel);
    }

    /// <summary>
    /// 获取最末级部门对应编号(最末3位)
    /// </summary>
    public static string GetLastOrgCode(this string orgCode)
    {
        if (orgCode.Length < 3)
            throw new UserFriendlyException("非法部门编码");
        return orgCode.Substring(orgCode.Length - 3, 3);
    }
}
